% Test code for longterm sinusoid removal
%
% Adaptive filtering techniques for the removal of noisy
% oscillatory transients from interferometric data
% E. Chassande-Mottin and S. V. Dhurandhar
%
% version: Wed Feb 16 18:34:22 MET 2000


clear

% GRAPHICS
printpath='./';
printok=1;

% SIGNAL
xcol=512; % 
fs=1000;  % sampling frequency
f0=50;    % transient freq.
A=1;      % signal amplitude
B=0.1;    % noise scaling
s=real(A*fmconst(xcol,f0/fs)*exp(i*pi/2));
n=B*randn(xcol,1);
t=(1:xcol)/fs;
x=s+n;

% ALGORITHM
d=5;        % prediction depth
N=40;       % number of tap-weight
mu=0.003;   % step gain parameter

[y wrecord]=adpt(x,mu,N,d,zeros(1,N),1);

% computing the R matrix
delta=1/fs;
deltap=2*pi*f0*delta;
v=d+(N-1:-1:0);
R=zeros(N);
for jcol=1:N,
  for kcol=1:N,
    R(jcol,kcol)=cos(deltap*(v(jcol)-v(kcol)))/2;
  end;
end;
R=R+B^2*eye(N);
p=cos(deltap*v')/2;
wstar=inv(R)*p;

% going to the eigen space v tilda
[Q,D]=eig(R);
D=diag(D);

vtrecord=zeros(xcol,N);
for jcol=1:xcol,
  vtrecord(jcol,:)=(wrecord(jcol,:)-wstar')*Q;
end;

% compute the EMSE
threshold=A^2*mu*B^2*N/2;
xis=zeros(xcol,1);
for jcol=1:xcol,
  D0=diag(D(N-1:N));
  vt=vtrecord(jcol,N-1:N)';
  xis(jcol)=vt'*D0*vt;;
end;
 
% theoretical value for t_lock
tlockthe=1/(2*fs)*log(mu*A^2*N*B.^2)/log(1-A^2*mu*N/2);


n1=N-1;
n2=N;
nplot=3;
tplot='(c)';
flag=0;
figure(1)
clf;

% plot filter trajectory in eigen-space
while(flag<2),
  Nj=20;
  vt1max=1.1*max(abs(vtrecord(:,n1)));
  vt2max=1.1*max(abs(vtrecord(:,n2)));
  vt1=linspace(-vt1max,vt1max,Nj);
  vt2=linspace(-vt2max,vt2max,Nj/2);
  J=zeros(Nj,Nj/2);
  D0=diag([D(n1) D(n2)]);
  for jrow=1:length(vt1),
    for kcol=1:length(vt2),
      vt=[vt1(jrow) vt2(kcol)]';
      J(jrow,kcol)=B^2+wstar'*p+vt'*D0*vt;
    end;
  end;
  subplot(2,2,nplot);
  Jmax=max(max(J));
  Jmin=min(min(J));
  Jv=round(100*linspace(1.1*Jmin,0.9*Jmax,4))/100;
  [c,h]=contour(vt2,vt1,J,Jv);
  clabel(c,h);
  hold on;
  plot(vtrecord(N+d:end,n2),vtrecord(N+d:end,n1));
  xplot=['v^{~}_{' num2str(N-n2+1)-1 '}'];
  yplot=['v^{~}_{' num2str(N-n1+1)-1 '}'];
  xlabel(xplot);ylabel(yplot);
  title(tplot);
  %
  n1=N-2; 
  n2=N-1;
  nplot=4;   
  tplot='(d)';
  flag=flag+1;
end;

subplot(2,2,1);
xmin=1.1*min(x);
xmax=1.1*max(x);
plot(t-t(N+d),s,'--');
hold on
plot(t-t(N+d),y);
hold off
grid on;
axis([0 0.2 xmin xmax]);
xlabel('time (s)');
title('(a)');

subplot(2,2,2);
ximax=max([xis]);
ximin=min([xis]);
semilogy(t-t(N+d),xis,'r');
hold on
semilogy([t(1)-t(N+d) t(xcol)-t(N+d)],[threshold threshold],'--g');
semilogy([tlockthe tlockthe],[ximin ximax],'--k');
hold off
axis([0 .2 ximin ximax]);
xlabel('time (s)');ylabel('excess mean error');
title('(b)');
    
if printok,
  eval(['print -dps ' printpath 'adapttest1.ps']);
end;