% Test code for longterm sinusoid removal
% Validation of t_lock theoretical value
%
% Adaptive filtering techniques for the removal of noisy
% oscillatory transients from interferometric data
% E. Chassande-Mottin and S. V. Dhurandhar
%
% version: Wed Feb 16 18:34:08 MET 2000


clear

% GRAPHICS
printpath='./';
graph=1;
printok=1;

% SIGNAL
xcol=2048;
fs=200;  % sampling frequency
f0=50;   % transient freq.
A=1;     % signal amplitude
s=real(A*fmconst(xcol,f0/fs));

t=(1:xcol)/fs;

% ALGORITHM

d=5;                 % prediction depth
N=200;               % number of tap-weight
mutilde=.005/N;
mu=mutilde/(A^2/2);  % step gain parameter


ntlock=20;
tlockcal=zeros(ntlock,1);
tlockthe =zeros(ntlock,1);
Bvect=logspace(-.5,-3,ntlock);

% loop for several sigma (B)
for tlockcol=1:ntlock,
   
  B=Bvect(tlockcol);
  n=B*randn(xcol,1);
  x=s+n;
  
  [y wrecord]=adpt(x,[mutilde 0],N,d,zeros(1,N),0);
  
  
  % computing the R matrix
  delta=1/fs;
  deltap=2*pi*f0*delta;
  v=d+(N-1:-1:0);
  R=zeros(N);
  for jcol=1:N,
    for kcol=1:N,
      R(jcol,kcol)=A^2*cos(deltap*(v(jcol)-v(kcol)))/2;
    end;
  end;
  R=R+B^2*eye(N);
  p=A^2*cos(deltap*v')/2;
  wstar=inv(R)*p;
  
  % going to the eigen space v tilda
  [Q,D]=eig(R);
  D=diag(D);
  
  vtrecord=zeros(xcol,N);
  for jcol=1:xcol,
    vtrecord(jcol,:)=(wrecord(jcol,:)-wstar')*Q;
  end;
  
  % compute the EMSE
  threshold=A^2*mu*B^2*N/2;
  xis=zeros(xcol,1);
  for jcol=1:xcol,
    D0=diag(D(N-1:N));
    vt=vtrecord(jcol,N-1:N)';
    xis(jcol)=vt'*D0*vt;;
  end;
  
  % measuring t_lock
  tmeasure=(min(find(xis<threshold))-(N+d))/fs;
  if ~isempty(tmeasure),
    tlockcal(tlockcol)=tmeasure;
  else
    tlockcal(tlockcol)=nan;
  end;
  
  % theoretical value for t_lock
  tlockthe(tlockcol)=1/(2*fs)*log(2*mutilde*N*B.^2)/log(1-mutilde*N);
  
  if graph,
    figure(1);
    ximax=max([xis]);
    ximin=min([xis]);
    semilogy(t,xis,'r');
    hold on
    semilogy([t(1) t(xcol)],[threshold threshold],'--g');
    semilogy((N+d)/fs+[tlockthe(tlockcol) tlockthe(tlockcol)],[ximin ximax],'--k');
    hold off
    axis([t(1) t(end) ximin ximax]);
    xlabel('time (s)');ylabel('excess mean error');
    drawnow
  end;  
end;

if graph,
  figure(2)
  subplot(1,2,1)
  semilogx(Bvect.^2,tlockcal,'r-+');
  hold on
  semilogx(Bvect.^2,tlockthe,'b');
  hold off
  xlabel('\sigma^2');ylabel('t_{lock} (seconds)');
  axis([Bvect(end).^2 Bvect(1).^2 min([min(tlockcal) ...
	  min(tlockthe)]) max([max(tlockcal) max(tlockthe)])]);
  grid on  
  drawnow
end;


% loop for several sigma (mu N)
ntlock=20;
N=200;
B=.1;
tlockcal=zeros(ntlock,1);
tlockthe =zeros(ntlock,1);

mutildeNmin=fzero([num2str(2*xcol) '*log(1-x)-log(x)-' num2str(log(2*B^2))],[eps 1/2]);
mutildeN=logspace(log10(mutildeNmin),log10(1),ntlock);

for tlockcol=1:ntlock,

  n=B*randn(xcol,1);
  x=s+n;

  mutilde=mutildeN(tlockcol)/N;
  [y wrecord]=adpt(x,[mutilde 0],N,d,zeros(1,N),0);
  mu=mutilde/(A^2/2);
  
  % computing the R matrix
  delta=1/fs;
  deltap=2*pi*f0*delta;
  v=d+(N-1:-1:0);
  R=zeros(N);
  for jcol=1:N,
    for kcol=1:N,
      R(jcol,kcol)=A^2*cos(deltap*(v(jcol)-v(kcol)))/2;
    end;
  end;
  R=R+B^2*eye(N);
  p=A^2*cos(deltap*v')/2;
  wstar=inv(R)*p;
  
  % going to the eigen space v tilda
  [Q,D]=eig(R);
  D=diag(D);
  
  vtrecord=zeros(xcol,N);
  for jcol=1:xcol,
    vtrecord(jcol,:)=(wrecord(jcol,:)-wstar')*Q;
  end;
  
  % compute the EMSE
  threshold=A^2*mu*B^2*N/2;
  xis=zeros(xcol,1);
  for jcol=1:xcol,
    D0=diag(D(N-1:N));
    vt=vtrecord(jcol,N-1:N)';
    xis(jcol)=vt'*D0*vt;;
  end;
  
  % measuring t_lock
  tmeasure=(min(find(xis<threshold))-(N+d))/fs;
  if ~isempty(tmeasure),
    tlockcal(tlockcol)=tmeasure;
  else
    tlockcal(tlockcol)=nan;
  end;
  
  % theoretical value for t_lock
  tlockthe(tlockcol)=1/(2*fs)*log(2*mutilde*N*B.^2)/log(1-mutilde*N);
  
  if graph,
    figure(1);
    ximax=max([xis]);
    ximin=min([xis]);
    semilogy(t,xis,'r');
    hold on
    semilogy([t(1) t(xcol)],[threshold threshold],'--g');
    semilogy((N+d)/fs+[tlockthe(tlockcol) tlockthe(tlockcol)],[ximin ximax],'--k');
    hold off
    axis([t(1) t(end) ximin ximax]);
    xlabel('time (s)');ylabel('excess mean error');
    drawnow
  end;
  
end;

if graph,
  figure(2)
  subplot(1,2,2)
  loglog(mutildeN,tlockcal,'r-+');
  hold on
  loglog(mutildeN,tlockthe,'b');
  hold off
  xlabel('\mu N A^2/2');ylabel('t_{lock} (seconds)');
  axis([mutildeN(1) mutildeN(end) min([min(tlockcal) ...
	  min(tlockthe)]) max([max(tlockcal) max(tlockthe)])]);
  grid on
end;

if printok,
  eval(['print -dps ' printpath 'adapttest1b.ps']);
end;
