% Test code for transient removal
%
% Adaptive filtering techniques for the removal of noisy
% oscillatory transients from interferometric data
% E. Chassande-Mottin and S. V. Dhurandhar
%
% version: Tue Dec 14 15:02:26 MET 1999
%
% RUN adaptsignal first!
%
% signal :  xorg
% sampling frequency: fs

% GRAPHICS
printpath='./';
printok=1;

% -----------------------------------------------------------
% OSCILLATORY TRANSIENT REMOVAL
% -----------------------------------------------------------

% Algorithm parameters
d=5;         % ALE: decorrelation delay
deltaf=3;    % ALE: minimum bandwidth authorized for the ALE filter 
             % (in Hz)
N=round(fs/deltaf);  % ALE: # of tap coef.
tlockdef=N;  % ALE: fixing the locking time 
P0=0.01;     % Probability for the distance to Gaussianity

% Internal parameters
lo=10;       % Length of the smoothing filter for the envelope
nfft=xcol/4; % # freq. bins for Welch PS estimate 
dh=25;       % spectrogram window length
h=amgauss(2*dh+1,dh,sqrt(xcol)); % spectrogram window

% Variable definitions
xfoutput=zeros(xcol,1);

% Welch PS and estimate sigma^2
[psf f]=welch(xorg,2*nfft,.5,window(xcol/4-1),fs);
devf=std(psf);
sigma2=mean(psf);
inlier=find(psf<(sigma2+2*devf));
sigma2=fs*mean(psf(inlier));

% estimate mu
mutildeN=fzero([num2str(2*tlockdef) '*log(1-x)-log(x)-' num2str(log(2*sigma2))],[eps 1/2]);
mu_train=mutildeN/N;
  
% run the ALE
[y w]=adpt(xorg,[mu_train 0],N,d,zeros(1,N),0);

% Gaussianity test on the envelope
envel=abs(hilbert(y)).^2;
envel1=conv(envel,ones(2*lo+1,1)/(2*lo+1));
envel=envel1(lo+1:end-lo);
threshold=chi2inv(1-P0,2*lo+1)*(2*sum(w'.^2)'.*sigma2)/(2*lo+1);

% remove ringdown when sufficient energy in the filtered output
index=(envel>threshold);
e=xorg;
e(index)=xorg(index)-y(index);

figure(1)
clf
t=(1:xcol)/fs;
tindex=2*(N+d)+2*dh:xcol;
xmax=max(abs(xorg(tindex)));

subplot(3,2,1);
plot(t(tindex),xorg(tindex));
grid;
hold off
axis([t(tindex(1)) t(tindex(end)) -xmax xmax]);
xlabel('time (s)');
ylabel('amplitude'); 
title('(a) input signal');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

fmin=0;
Nf=256;
ftf=fmin+(0:Nf/2-1)/Nf*fs;
ttf=(tindex(1):round(length(tindex)/256):tindex(end));

subplot(3,2,2); 
tfr=tfrsp(xorg,ttf,Nf,h);
imagesc(ttf/fs,ftf,tfr(1:Nf/2,:));
axis('xy');
xlabel('time (s)');
ylabel('frequency (Hz)'); 
title('(b)');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

subplot(3,2,3);
emax=1.1*max(envel(tindex));
plot(t(tindex),envel(tindex));
hold on
plot(t(tindex),emax/1.1*index(tindex),'.k');
plot(t(tindex),threshold(tindex),'r--');
hold off
grid;
axis([t(tindex(1)) t(tindex(end)) 0 emax]);
xlabel('time (s)');
ylabel('envelope'); 
title('(c) filtered ouput');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

subplot(3,2,5);
plot(t(tindex),e(tindex));
grid;
hold off
axis([t(tindex(1)) t(tindex(end)) -xmax xmax]);
xlabel('time (s)');
ylabel('amplitude'); 
title('(d) output signal');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

subplot(3,2,4);
tindex=round(linspace(2*(N+d)+2*dh,xcol,200));
wfm=abs(fft([w(tindex,:) zeros(200,256-N)]'));
select=ones(128,1)*index(tindex)';
wmat=abs(wfm(1:128,:));
wmat=select.*wmat;
imagesc(tindex/fs,fmin+fs*(0:128)/256,wmat);
xlabel('time (s)');
ylabel('frequency (Hz)'); 
title('(d) adaptive filter');
axis('xy');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

fmin=0;
Nf=256;
ftf=fmin+(0:Nf/2-1)/Nf*fs;
ttf=(tindex(1):round(length(tindex)/256):tindex(end));

subplot(3,2,6); 
tfr=tfrsp(e,ttf,Nf,h);
imagesc(ttf/fs,ftf,tfr(1:Nf/2,:));
axis('xy');
xlabel('time (s)');
ylabel('frequency (Hz)'); 
title('(e)');
pos=get(gca,'Position');
set(gca,'Position',pos*diag([1 1 1 4/5]));

clear tindex wfm select wmat
drawnow

if (printok==1),
  colormap(flipud(gray));
   eval(['print -dps ' printpath 'adapttest2.ps']);
end;