function [y,w]=adpt(x,p0,N,d,wstart,trace);
%ADPT     Adaptive Line Enhancement
%
%          [y w]=adpt(x,p0,N,d,trace)
% 
%          x: input signal
%         p0: iteration step parameter
%         first case  p0=[mu] (default case with : p0=0.01) 
%         constant step
%         second case  p0=[mutilde flg]
%         normalized step gain parameter.
%         third case p0=[mub mue mut]
%         exp. decreasing step from mub to mue with a typical decay 
%         time of mut
%          N: number of coefficients of the adaptive filter
%             (default: N=20)
%          d: prediction delay (default: d=10)
%      trace: if nonzero, the progression of the 
%             algorithm is shown (default: trace=0)
%
%          y: output (filtered) signal
%          w: filter matrix (collection of all the filters)
% 
% Example:
% s=fmconst(256,0.1);n=randn(256,1);x=sigmerge(s,n,10);
% [y,w]=adpt(x);
%      
% Eric Chassande-Mottin, eric@aei-postdam.mpg.de
% Sanjeev Dhurandar, sanjeev@aei-postdam.mpg.de
% 20/03/99

[xrow xcol]=size(x);
if xcol~=1,
  error('X must be a column vector');
end;

if (nargin == 1),
  p0=0.01; N=20; d=10; wstart=zeros(1,N); trace=0;
elseif (nargin == 2),
  N=20; d=10;  wstart=zeros(1,N); trace=0;
elseif (nargin == 3),
  d=10;  wstart=zeros(1,N); trace=0;
elseif (nargin == 4),
  wstart=zeros(1,N); trace=0;
elseif (nargin == 5),
  trace=0;
end;

[wrow wcol]=size(wstart);
if (wrow~=1)|(wcol~=N),
  error('WSTART must be a (1,N) vector');
end;

if length(p0)==1,
  muflg=0; % const. step
  mu=p0; 
elseif length(p0)==2,
  muflg=1; % normalized step
  mutilde=p0(1);
elseif length(p0)==3,
  muflg=2; % exp. decreasing step
  mub=p0(1);
  mue=p0(2);
  mut=p0(3);
  mua=log(0.1*mue/(mub-mue))/mut;
end;

fres=10; % maximum value for the filter coefficients

xft=zeros(N,1);
y=zeros(xrow,1);
w=zeros(xrow,N);
w(1:N+d-1,:)=ones(N+d-1,1)*wstart;

if trace,
  fprintf('ALE\n');
end;

for jcol=(N+d):xrow,
  if trace,
    disprog(jcol,xrow,10);
  end;
  xft=x(jcol-d-N+1:jcol-d);
  % filter sig.
  y(jcol)=w(jcol-1,:)*xft;
  % compute the error
  e(jcol)=x(jcol)-y(jcol);
  % step
  if muflg==1,
    mu=mutilde/mean(abs(xft).^2);
  elseif muflg==2,
    mu=mue+(mub-mue)*exp(mua*(jcol-(N+d)+1));
  end;
  % compute next coef. filters
  w(jcol,:)=w(jcol-1,:)+2*mu*e(jcol)*xft';
  % stabilization (to prevent cases of divergence)
  % if one of the coef. value happens to be to big then
  % keep the previous value
  if  any(abs(w(jcol,:))>fres),
    index=find(abs(w(jcol,:))>fres);
    w(jcol,index)= w(jcol-1,index);
  end;
end;