function [data, fgetoutput, response]=gr40mload(npoint,fmin,fmax)
% GR40MLOAD loads Caltech 40m data (in Frame format)
% and normalize them, given sweep sine calibration data. 
% The output signal is the h-reconstructed signal (dl/L).
%
%        [data, srate, response]=gr40mload(npoint);
%
%           npoint: number of points to download
%
%             data: normalized output strain signal
%       fgetoutput: structure containing non processed data and
%                   book keeping run information
%             resp: frequency normalization
%
% Example: 
% [data fgetoutput]=gr40mload; 
% srate=fgetoutput.srate; npoint=fgetoutput.npoint;
% t=(1:npoint)/srate;
% plot(t,data); xlabel('time (s)');
%
% Rem:
% starting time is chosen by the computer.
% fgetoutput.srate: sampling frequency
%
% Eric Chassande-Mottin, Wed Feb 23 19:42:50 MET 2000
% eric@aei-potsdam.mpg.de
%
% This function uses GRtool mex functions (mxFget_ch and mxGRnormalize)
% developped by Steve Drasco, and associated GRASP and FRAME routines.

if (nargin == 0),
  xcol=655536;
  fmin=0;
  fmax=0;
elseif (nargin == 1),
  fmin=0;
  fmax=0;
elseif (nargin == 2),
  fmax=0;
end;    
    
ARMLENGTH_1994=38.25;

% set the structure values
fgetinput.nchan = 1;
fgetinput.chnames = {'IFO_DMRO'};
fgetinput.inlock = 1;
fgetinput.npoint = npoint;
fgetinput.calibrate = 1;

% get the data
fgetinput.seek = 0;
fgetoutput = mxFget_ch(fgetinput);
srate = fgetoutput.srate;

% Fourier transform the data
dataft = fft(fgetoutput.data{1});

% one-sided power-spectrum normalization 
response=mxGRnormalize(fgetoutput.fri,fgetoutput.frinum,npoint,srate);

% real and imaginary parts of R (recontructed from realft standards)
resp= response(1+2*(2:npoint/2))+i*response(2+2*(2:npoint/2));

% normalization
normdataft= zeros(npoint,1);
normdataft(2:npoint/2)= resp .* dataft(2:npoint/2)/ARMLENGTH_1994;

% selection of the data between fmin and fmax
if (fmax==0),
  fmax=srate/2;
end;
f=(0:npoint/2-1)/npoint*srate;
indexmin=min(find(f>fmin));
indexmax=min(find(f>fmax));
normdataft(1:indexmin-1)=zeros(indexmin-1,1);
normdataft(indexmax+1:npoint/2)=zeros(npoint/2-indexmax,1);

% Inverse Fourier transform
data=2*real(ifft(normdataft));

% [p,f]=psd(data,4096,srate);
% loglog(f,factor*sqrt(p));
% axis([50 5000 1e-19 1e-14]);
% grid