% Code for line and transient removal
%
% Adaptive filtering techniques for the removal of noisy
% oscillatory transients from interferometric data
% E. Chassande-Mottin and S. V. Dhurandhar
%
% version: Tue Dec 14 15:02:26 MET 1999

% Run rmsignal before!
% signal 1:  xorg1 (training data)
% signal 2:  xorg2 (actual data)
% sampling frequency: fs

% GRAPHICS
trace=0;      % process shown (yes=1/no=0)
graph=1;      % graphics shown (yes=1/no=0)
printok=1;    % print figures (yes=1/no=0)
jbandprint=5; % Band index for printing
printpath='./';

% -----------------------------------------------------------
% FIXED FREQUENCY REMOVAL
% -----------------------------------------------------------

% Algorithm parameters
Nband=32;        % # of subbands (has to be a power of 2)
d=5;             % ALE: decorrelation delay
pcnoise=1;       % percentage of noise in the filtered output
N=2*100/pcnoise; % ALE: # of tap coef.
pcsig=1;         % percentage of signal in the error output
muinit=1/(2*N);  % training: mu_initial (in the training)
mucol=4;         % training: # of passes (with decreasing mu)

% Internal parameters
xcolband=xcol/Nband;
fsband=fs/Nband;
nfft=xcolband/4; % # of freq. bins for pow. spect. estimation
dh=25;       % spectrogram window length
h=amgauss(2*dh+1,dh,sqrt(xcolband)); % spectrogram window

% Variable definitions
xfoutput1=zeros(xcol,1);
xfoutput2=zeros(xcol,1);
wstart=zeros(Nband,N);
muband=zeros(Nband,1);

% TRAINING AND CLEANING THE TRAINING DATA

xf=fft(xorg1);

for jband=1:Nband,

  % get the time signal in the subband (demodulation)
  indexmin=1+(jband-1)*xcolband/2;
  indexmax=indexmin+xcolband/2-1;
  xf2=zeros(xcolband,1);
  xf2(1:xcolband/2)=xf(indexmin:indexmax);
  xband=2*real(ifft(xf2));
  clear xf2

  if trace
    fprintf('Band # %2.0f: fmin=%4.0f Hz -> fmax=%4.0f Hz\n',jband,indexmin/xcol*fs,indexmax/xcol*fs);
  end;
  
  % estimate parameters from signal
  % compute the PSD
  [psf f]=welch(xband,2*nfft,.5,window(xcolband/4-1),fsband);
  
  % estimate sigma^2
  devf=std(psf);
  sigma2=mean(psf);
  inlier=find(psf<(sigma2+2*devf));
  sigma2=fsband*mean(psf(inlier));
  
  % and A
  freind=find(psf==max(psf));
  freind=max(freind-1,1):min(freind+1,nfft);
  A2=4*fsband*sum(psf(freind))/nfft;
    
  if graph,
    figure(1);
    clf;
    plot(f,psf);title('Power spectrum (ADC unit)');ylabel('frequency (Hz)');
    hold on; plot([f(1) f(end)],[sigma2 sigma2]/fsband,'--r');
    plot([f(1) f(end)],[A2 A2]/fsband,'--g'); hold off;
    grid;set(gca,'xtick',0:5:160)
  end;
  
  % verify if the selected peak is (at least) SNR=1
  if (A2>sigma2),

    if trace,
      fprintf('fixed frequency removal:');
    end;
   
    % compute mu_final
    muband(jband)=A2/(2*N*sigma2)*pcsig/100;
        
    %fprintf('band = %2.0f: A= %1.3E, s= %1.3E muN=%1.4f\n',jband,sqrt(A2),sqrt(sigma2),N*muband(jband));
    
    % choose mu sequence from init to final
    dmu=muband(jband)-muinit;
    mu= muinit+linspace(0,dmu,mucol);
 
    % run ALE
    xcolpass=xcolband/mucol;
    
    for jmu=1:mucol,
      if trace,
	fprintf(' pass #%2.0f -',jmu);
      end;
      indpassmin=1+(xcolpass)*(jmu-1);
      indpassmax=(xcolpass)*jmu;
      xpass=xband(indpassmin:indpassmax);
      [y w]=adpt(xpass,[mu(jmu) 0],N,d,wstart(jband,:),0);
      wstart(jband,:)=w(xcolpass,:);
    end;
    if trace,
      fprintf(' done.\n');
    end;
  else
    if trace,
      fprintf('No sufficient peak detected.\n');
    end;
    muband(jband)=0;
    y=zeros(xcolpass,1);
    indpassmin=1+(xcolpass)*(mucol-1);
    indpassmax=(xcolpass)*mucol;
    xpass=xband(indpassmin:indpassmax);
  end;
  
  % display pass #4
  if graph,
    figure(2)
    clf;
    
    t=(1:xcolpass)/fsband;
    subplot(2,2,1);
    index=((N+d)+2*dh):xcolpass;
    plot(t(index),xpass(index));
    grid;
    xmax=1.1*max(abs(xpass));
    axis([t(index(1)) t(index(end)) -xmax xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(a) input signal, band #' num2str(jband)]);
    
    subplot(2,2,3);
    e=xpass-y;
    plot(t(index),e(index));
    grid;
    axis([t(index(1)) t(index(end)) -xmax xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(c) output signal, band #' num2str(jband)]);

    subplot(2,2,2);
    fmin=indexmin/xcol*fs;
    Nf=256;
    ftf=fmin+(0:Nf/2-1)/Nf*fsband;
    ttf=(index(1):round(length(index)/256):index(end));
    tfr=tfrsp(xpass,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(b)');
  
    subplot(2,2,4);
    tfr=tfrsp(e,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)'); 
    ylabel('frequency (Hz)'); 
    title('(d)');;
    drawnow;
  
    % pause;
  end;

  % CLEANING THE TRAINING DATA
  if muband(jband)~=0,
    [y w]=adpt(xband,[muband(jband) 0],N,d,wstart(jband,:),0);

    if trace,
      fprintf('cleaning done.\n');
    end;
    e=xband-y;
  else
    e=xband;
  end;
  
  if graph,

    figure(1)
    clf
    t=(1:xcolband)/fsband;
    subplot(2,2,1);
    index=(2*(N+d)+2*dh):xcolband;
    plot(t(index),xband(index));
    grid;
    xmin=1.1*min(xband(index));
    xmax=1.1*max(xband(index));
    axis([t(index(1)) t(index(end)) xmin xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(a) input signal, Band #' num2str(jband)]);
    
    subplot(2,2,3);
    plot(t(index),e(index));
    grid;
    axis([t(index(1)) t(index(end)) xmin xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(c) output signal, band #' num2str(jband)]);
    
    subplot(2,2,2);
    fmin=indexmin/xcol*fs;
    Nf=256;
    ftf=fmin+(0:Nf/2-1)/Nf*fsband;
    ttf=index(1):round(length(index)/256):index(end);
    tfr=tfrsp(xband,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(b)');
    
    subplot(2,2,4);
    tfr=tfrsp(e,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(d)');
    
    %drawnow
    %pause;
  end;

  drawnow;

  % recombine the data
  xf2=fft(e);
  if jband~=1,
    xfoutput1(indexmin:indexmax)=xf2(1:xcolband/2);
  else
    if trace,
      fprintf('Warning ! Set output to 0 in Band #1 !!\n');
    end;
  end;  
  
  indexmin=indexmin+xcolband/2;

end;

clear tfr w

% output of the first step (training data)
xoutput1=2*real(ifft(xfoutput1));
xoutput1=[zeros((2*(N+d))*Nband-1,1);xoutput1((2*(N+d))*Nband:end)];

clear tfr w xf

% CLEANING THE ACTUAL DATA

xf=fft(xorg2);

for jband=1:Nband,
  
  % get the time signal in the subband (demodulation)
  indexmin=1+(jband-1)*xcolband/2;
  indexmax=indexmin+xcolband/2-1;
  xf2=zeros(xcolband,1);
  xf2(1:xcolband/2)=xf(indexmin:indexmax);
  xband=2*real(ifft(xf2));
  clear xf2

  if trace
    fprintf('Band # %2.0f: fmin=%4.0f Hz -> fmax=%4.0f Hz\n',jband,indexmin/xcol*fs,indexmax/xcol*fs);
  end;

  % CLEANING THE ACTUAL DATA
  if muband(jband)~=0,
    [y w]=adpt(xband,[muband(jband) 0],N,d,wstart(jband,:),0);
    
    if trace,
      fprintf('cleaning done.\n');
    end;
    e=xband-y;
  else
    e=xband;
  end;
  
    
  if graph,

    figure(1)
    clf
    t=(1:xcolband)/fsband;
    subplot(2,2,1);
    index=(2*(N+d)+2*dh):xcolband;
    plot(t(index),xband(index));
    grid;
    xmax=1.1*max(abs(xband(index)));
    axis([t(index(1)) t(index(end)) -xmax xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(a) input signal, band #' num2str(jband)]);
    
    subplot(2,2,3);
    plot(t(index),e(index));
    grid;
    axis([t(index(1)) t(index(end)) -xmax xmax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(c) output signal, band #' num2str(jband)]);
    
    subplot(2,2,2); 
    fmin=indexmin/xcol*fs;
    Nf=256;
    ftf=fmin+(0:Nf/2-1)/Nf*fsband;
    ttf=index(1):round(length(index)/256):index(end);
    tfr=tfrsp(xband,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(b)');
    
    subplot(2,2,4);
    tfr=tfrsp(e,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(d)');
    
    drawnow
    
    if (graph==1)&(printok==1)&(jband==jbandprint),
      colormap(flipud(gray));
      eval(['print -dps ' printpath 'step1fig.ps']);
      colormap(jet);
    end;
  end;

  % recombine data
  xf2=fft(e);
  if jband~=1,
    xfoutput2(indexmin:indexmax)=xf2(1:xcolband/2);
  else
    if trace,
      fprintf('Warning ! Set output to 0 in Band #1 !!\n');
    end;
  end;  
  
end;

% output of the first step (actual data)
xoutput2=2*real(ifft(xfoutput2));
xoutput2=[zeros((2*(N+d))*Nband-1,1);xoutput2((2*(N+d))*Nband:end)];

save step1 xoutput1 xoutput2

clear tfr w xf xoutput1 xoutput2

% -----------------------------------------------------------
% OSCILLATORY TRANSIENT REMOVAL
% -----------------------------------------------------------

% Algorithm parameters
d=5;         % ALE: decorrelation delay
deltaf=3;    % ALE: minimum bandwidth authorized for the ALE filter 
             % (in Hz)
N=round(fs/Nband/deltaf);  % ALE: # of tap coef.
tlockdef=N;  % ALE: fixing the locking time 
P0=0.01;     % Probability for the distance to Gaussianity

% Internal parameters
lo=10;       % Length of the smoothing filter for the envelope

% Variable definitions
xfoutput=zeros(xcol,1);
wstart=zeros(Nband,N); 

% TRAINING ON THE TRAINING DATA

for jband=2:Nband,
  
  % get the time signal in the subband (demodulation)
  indexmin=1+(jband-1)*xcolband/2;
  indexmax=indexmin+xcolband/2-1;
  xf2=zeros(xcolband,1);
  xf2(1:xcolband/2)=xfoutput1(indexmin:indexmax);
  xband=2*real(ifft(xf2));
  clear xf2;
 
  if trace,
    fprintf('Band # %2.0f: fmin=%4.0f Hz -> fmax=%4.0f Hz\n',jband,indexmin/xcol*fs,indexmax/xcol*fs);
  end;

  % Welch PS and estimate sigma^2
  [psf f]=welch(xband,2*nfft,.5,window(xcolband/4-1),fsband);
  devf=std(psf);
  sigma2=mean(psf);
  inlier=find(psf<(sigma2+2*devf));
  sigma2=fsband*mean(psf(inlier));
  
  % estimate mu
  mutildeN=fzero([num2str(2*tlockdef) '*log(1-x)-log(x)-' num2str(log(2*sigma2))],[eps 1/2]);
  mu_train=mutildeN/N;
  %fprintf('mu_train=%1.4f:\n',mu_train);
  
  % run the ALE
  if trace,
    fprintf('Step 2:');
  end;
  [y w]=adpt(xband,[mu_train 0],N,d,wstart(jband,:),0);
  wstart(jband,:)=w(xcolband,:);
  if trace,
    fprintf(' done.\n');
  end;
  
  if graph,
    
    % Gaussianity test on the envelope
    envel=abs(hilbert(y)).^2;
    envel1=conv(envel,ones(2*lo+1,1)/(2*lo+1));
    envel=envel1(lo+1:end-lo);
    threshold=chi2inv(1-P0,2*lo+1)*(2*sum(w'.^2)'.*sigma2)/(2*lo+1);
    
    % remove ringdown when sufficient energy in the filtered output
    index=(envel>threshold);
    e=xband;
    e(index)=xband(index)-y(index);
    
    figure(2)
    clf
    t=(1:xcolband)/fsband;
    tindex=2*(N+d)+2*dh:xcolband;
    emax=max(abs(e(tindex)));
    
    subplot(2,2,1);
    emax=1.1*max(envel(tindex));
    plot(t(tindex),envel(tindex));
    hold on
    plot(t(tindex),emax/1.1*index(tindex),'.k');
    plot(t(tindex),threshold(tindex),'r--');
    hold off
    grid;
    axis([t(tindex(1)) t(tindex(end)) 0 emax]);
    xlabel('time (s)');
    ylabel('envelope'); 
    title('(e) filtered ouput');
    
    subplot(2,2,3);
    emax=max(abs(e(tindex)));
    plot(t(tindex),e(tindex));
    grid;
    axis([t(tindex(1)) t(tindex(end)) -emax emax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(g) output signal,  band #' num2str(jband)]);
    
    fmin=indexmin/xcol*fs;
    Nf=256;
    ftf=fmin+(0:Nf/2-1)/Nf*fsband;
    ttf=(tindex(1):round(length(tindex)/256):tindex(end));

    subplot(2,2,4); 
    tfr=tfrsp(e,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(h)');
    
    subplot(2,2,2);
    tindex=round(linspace(2*(N+d)+2*dh,xcolband,200));
    wfm=abs(fft([w(tindex,:) zeros(200,256-N)]'));
    select=ones(128,1)*index(tindex)';
    wmat=abs(wfm(1:128,:));
    wmat=select.*wmat;
    imagesc(tindex/fsband,fmin+fsband*(0:128)/256,wmat);
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(f) adaptive filter');
    axis('xy');
    
    clear tindex wfm select wmat
    drawnow
    
  end;  
end;

clear  tfr x w

% CLEANING ON ACTUAL DATA

for jband=2:Nband,

  % get the time signal in the subband (demodulation)
  indexmin=1+(jband-1)*xcolband/2;
  indexmax=indexmin+xcolband/2-1;
  xf2=zeros(xcolband,1);
  xf2(1:xcolband/2)=xfoutput2(indexmin:indexmax);
  xband=2*real(ifft(xf2));
  clear xf2

  if trace,
    fprintf('Band # %2.0f: fmin=%4.0f Hz -> fmax=%4.0f Hz\n',jband,indexmin/xcol*fs,indexmax/xcol*fs);
  end;
  
  % Welch PS and estimate sigma^2
  [psf f]=welch(xband,2*nfft,.5,window(xcolband/4-1),fsband);
  devf=std(psf);
  sigma2=mean(psf);
  inlier=find(psf<(sigma2+2*devf));
  sigma2=fsband*mean(psf(inlier));

  % estimate mu
  mutildeN=fzero([num2str(2*tlockdef) '*log(1-x)-log(x)-' num2str(log(2*sigma2))],[eps 1/2]);
  mu_final=mutildeN/N;

  %fprintf('band = %2.0f: s= %1.3E muN=%1.4f\n',jband,sqrt(sigma2),muband(jband)*N);
  
  % run the ALE
  if trace,
    fprintf('Step 2:');
  end;
  [y w]=adpt(xband,[mu_final 0],N,d,wstart(jband,:),0);
  if trace,
    fprintf('done.\n');
  end;
  
  % Gaussianity test on the envelope
  envel=abs(hilbert(y)).^2;
  envel1=conv(envel,ones(2*lo+1,1)/(2*lo+1));
  envel=envel1(lo+1:end-lo);
  threshold=chi2inv(0.99,2*lo+1)*(2*sum(w'.^2)'.*sigma2)/(2*lo+1);
  
  % remove ringdown when sufficient energy in the filtered output
  index=(envel>threshold);  
  e=xband;
  e(index)=xband(index)-y(index);
  
  if graph,
    
    figure(1)
    clf
    t=(1:xcolband)/fsband;
    tindex=2*(N+d)+2*dh:xcolband;

    subplot(2,2,1);
    emax=1.1*max(envel(tindex));
    plot(t(tindex),envel(tindex));
    hold on
    plot(t(tindex),emax/1.1*index(tindex),'.k');
    plot(t(tindex),threshold(tindex),'r--');
    hold off
    grid;
    axis([t(tindex(1)) t(tindex(end)) 0 emax]);
    xlabel('time (s)');
    ylabel('envelope'); 
    title('(e) filtered ouput');
    
    subplot(2,2,3);
    emax=max(abs(e(tindex)));
    plot(t(tindex),e(tindex));
    grid;
    axis([t(tindex(1)) t(tindex(end)) -emax emax]);
    xlabel('time (s)');
    ylabel('amplitude'); 
    title(['(g) output signal,  band #' num2str(jband)]);
    
    fmin=indexmin/xcol*fs;
    Nf=256;
    ftf=fmin+(0:Nf/2-1)/Nf*fsband;
    ttf=(tindex(1):round(length(tindex)/256):tindex(end));

    subplot(2,2,4); 
    tfr=tfrsp(e,ttf,Nf,h);
    imagesc(ttf/fsband,ftf,tfr(1:Nf/2,:));
    axis('xy');
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(h)');
    
    subplot(2,2,2);
    tindex=round(linspace(2*(N+d)+2*dh,xcolband,200));
    wfm=abs(fft([w(tindex,:) zeros(200,256-N)]'));
    select=ones(128,1)*index(tindex)';
    wmat=abs(wfm(1:128,:));
    wmat=select.*wmat;
    imagesc(tindex/fsband,fmin+fsband*(0:128)/256,wmat);
    xlabel('time (s)');
    ylabel('frequency (Hz)'); 
    title('(f) adaptive filter');
    axis('xy');

    clear tindex wfm select wmat
    drawnow
    
    if (graph==1)&(printok==1)&(jband==jbandprint),
      colormap(flipud(gray));
      eval(['print -dps ' printpath 'step2fig.ps']);
      colormap(jet);
    end;
    
  end;

  % recombine data
  xf2=fft(e);
  xfoutput(indexmin:indexmax)=2*xf2(1:xcolband/2);
  
end;

% output of the second step (actual data)
xoutput=real(ifft(xfoutput));
xoutput=[zeros((2*(N+d))*Nband-1,1);xoutput((2*(N+d))*Nband:end)];

save step2 xoutput fs xcol

