function [ps,f,tfr] = welch(x,N,olp,h,fs,unit,trace);
%WELCH  Power Spectrum Estimation with Welch method.
%	[PS,F,TFR]=WELCH(X,N,OLP,H,FS,UNIT,TRACE) computes the Welch
%	estimator of the power spectrum of the signal X. 
% 
%	X     : signal.
%	N     : number of frequency bins (default : length(X)).
%	OLP   : overlap between two successive data segments (default : .5).
%       FS    : sampling frequency (default : 1)
%       UNIT  : physical unit of the input signal (default : 'U') 
%	H     : analysis window, H being normalized so as to
%	        be  of unit energy.      (default : Hamming(N/4)). 
%	TRACE : if nonzero, the progression of the algorithm is shown
%	        (default : 0)
%
%       PS    : power spectrum estimate. When called without
%               argument, PS is displayed on the current figure.
%	TFR   : periodograms used to compute PS.
%	F     : vector of frequencies.
%
% 

%	E.Chassande-Mottin, eric@aei-potsdam.mpg.de
%       Wed Dec 15 19:02:44 MET 1999
%       Ref: Digital Signal Processing, Proakis & Manolakis
%       p. 911
%
%  Get the heuristic estimator of the pow. spec. with WELCH
%  N=1024; sig=randn(N,1); h=ones(N-1,1); 
%  [ps,f]=welch(sig,N,0,h,1);
%  ps2=abs(fft(sig).^2)/(N-1);
%  plot(f,ps,'r');hold on;plot(f,ps2(1:N/2));hold off;
%
%  You can see a non significant difference between PS and PS2
%  because of a wrapping used internally in WELCH
%  
%  Taken from Matlab's PSD function:
%
%  The units on the power spectra PS are such that, using
%  Parseval's theorem: 
%
%  SUM(PS)/LENGTH(PS) = SUM(X.^2)/LENGTH(X) = COV(X)
%
%  Here are the covariance, RMS, and spectral amplitude values of
%  some common functions:
%
%          Signal   Cov=SUM(PS)/LENGTH(PS)    RMS         PS
%          a*sin(w*t)        a^2/2            a/sqrt(2)   a^2*LENGTH(PS)/2
% Normal:  a*randn(t)        a^2              a           a^2
% Uniform: a*rand(t)         a^2/12           a/sqrt(12)  a^2/12
%   
%

[xrow,xcol] = size(x);
if (nargin < 1),
  error('At least 1 parameter is required');
elseif (nargin<2),
  N=xrow;
end;

hlength=floor(N/4);
hlength=hlength+1-rem(hlength,2);

if (nargin == 1),
  olp=0.5; h = window(hlength); fs=1; unit='U'; trace=0;
elseif (nargin == 2),
  olp=0.5; h = window(hlength); fs=1; unit='U'; trace=0 
elseif (nargin == 3),
  h = window(hlength);  fs=1; unit='U'; trace=0;
elseif (nargin == 4),
  fs=1; unit='U'; trace = 0;
elseif (nargin == 5),
  unit='U'; trace = 0;
elseif (nargin == 6),
  trace = 0;  
end;

if (N<0),
  error('N must be greater than zero');
end;

[olprow olpcol]=size(olp);
if (xcol~=1),
  error('X must have one column');
elseif (olpcol~=1)|(olprow~=1),
   error('OLP must be a scalar number');
 elseif (olp<=0)&(olp>=1),
   error('OLP must be between 0 and 1');
elseif (2^nextpow2(N)~=N),
  fprintf('For a faster computation, N should be a power of two\n')
end;

[hrow,hcol]=size(h); 
if (hcol~=1)|(rem(hrow,2)==0),
  error('H must be a smoothing window with odd length');
elseif (hrow>N),
  error('The window H is too long (LENGTH(H) must be smaller than N)');
end;
Lh=(hrow-1)/2;
hnorm=sqrt(sum(h.^2)/hrow);
h=h./hnorm;

holp=floor(hrow*olp);
tcol=floor(1+(xrow-hrow)/(hrow-holp));
t=Lh+1+(0:tcol-1)*(hrow-holp);

%fprintf('overlap size : %d \n',holp);
%fprintf('n. of windows: %d \n',tcol);
%fprintf('wind. center pos.: %2.0f \n',t)
%keyboard;

tfr= zeros (N,tcol) ;  
if trace, disp('Welch Power Spectrum estimator'); end;
for icol=1:tcol,
  ti= t(icol); tau=-min([round(N/2)-1,Lh]):min([round(N/2)-1,Lh]);
  indices= rem(N+tau,N)+1;
  if trace, disprog(icol,tcol,10); end;
  tfr(indices,icol)=x(ti+tau).*conj(h(Lh+1+tau));
end;
tfrnorm=hrow*fs;
tfr=abs(fft(tfr)).^2/tfrnorm; 

if tcol>1,
  ps=sum(tfr')'/tcol;
else
  ps=tfr;
end;

if trace, fprintf('\n'); end;

if fs<1000, unitHz=1;                            % display in Hz
elseif (fs>=1e3 & fs<1e6), fsg=fs/1e3; unitHz=2;  % display in kHz
elseif (fs>=1e6), fsg=fs/1e6; unitHz=3;           % display in MHz
end

if isreal(x),
  f=(0:N/2-1)/N;
  ps=ps(1:N/2);
  tfr=tfr(1:N/2,:);  
else
  f=(-N/2+1:N/2)/N;
end;

if (nargout==0),
  clf;
  if isreal(x),
    semilogy(f*fsg,ps);
    grid on;
  else
    semilogy(f*fsg,fftshift(ps));
    grid on;
  end

  ylabel(['Power Spectrum [' unit '^2/Hz]']);
  
  if unitHz==1,
    xlabel('Frequency [Hz]');
  elseif unitHz==2,
    xlabel('Frequency [kHz]');
  elseif unitHz==3,
    xlabel('Frequency [MHz]');
  end;
end;

f=f*fs;